# -*- coding: utf-8 -*-
## Copyright (C) 2007 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

from cStringIO import StringIO
from Products.CMFCore.DirectoryView import addDirectoryViews
from Products.CMFCore.utils import getToolByName
from Products.TagCloudExplorer import config
from Products.TagCloudExplorer import TOOL

import string

tceportlets = ['/here/%s/macros/portlet' % portlet for portlet in config.PORTLETS]

def installSubSkin(context, skinFolder, outStream):
    """
        Install a subskin in portal_skins
    """
    skinsTool = getToolByName(context, 'portal_skins')
    for skin in skinsTool.getSkinSelections() :
        path = skinsTool.getSkinPath(skin)
        path = map(string.strip, string.split(path, ','))
        if not skinFolder in path:
            try:
                path.insert(path.index('custom') + 1, skinFolder)
            except ValueError:
                path.append(skinFolder)
            path = string.join(path, ', ')
            skinsTool.addSkinSelection(skin, path)
            outStream.write('Subskin successfully installed into %s.\n' % skin)
        else:
            outStream.write('*** Subskin was already installed into %s.\n' % skin)

def addTool(self,out) :
    # Check that the tool has not been added using its id
    if not hasattr(self, TOOL.id) :
        addTool = self.manage_addProduct[config.PROJECT_NAME].manage_addTool
        # Add the tool by its meta_type
        addTool(TOOL.meta_type)
        out.write("Added %s tool\n" %TOOL.id)

def addActionProvider(self, out):
    pass

def setupCSS(self, out):
    # Register dplpegase css
    ctool = getToolByName(self, 'portal_css')
    ctool.registerStylesheet(config.CSS_MAIN)
    print >> out, 'CSS installed'
    
def installConfiglet(self, out) :
    configTool = getToolByName(self, 'portal_controlpanel', None)
    if configTool:
        for conf in config.configlets:
            configTool.registerConfiglet(**conf)
            out.write('Added configlet %s\n' % conf['id'])

def uninstallConfiglet(self, out) :
    configTool = getToolByName(self, 'portal_controlpanel', None)
    if configTool:
        for conf in config.configlets:
            configTool.unregisterConfiglet(conf['id'])
            out.write('Removed configlet %s\n' % conf['id'])

def addPortlet(self, out):
    """ add portlet to right slots """
    
    utool = getToolByName(self, 'portal_url')
    portal = utool.getPortalObject()
    portlets = list(portal.getProperty('right_slots'))
    
    for portlet in tceportlets:
        if portlet not in portlets:
            portlets.insert(0, portlet)
    
    portal._updateProperty('right_slots', portlets)
    
    print >> out, 'Cached portlet added.\n'

def removePortlet(self, out):
    """ remove portlet from right slots """
    
    utool = getToolByName(self, 'portal_url')
    portal = utool.getPortalObject()
    rportlets = list(portal.getProperty('right_slots'))
    
    for portlet in tceportlets:
        if portlet in rportlets:
            rportlets.remove(portlet)
            print >> out, '%s portlet removed.\n' % portlet
    
    portal._updateProperty('right_slots', rportlets)

    lportlets = list(portal.getProperty('left_slots'))
    
    for portlet in tceportlets:
        if portlet in lportlets:
            lportlets.remove(portlet)
            print >> out, '%s portlet removed.\n' % portlet
    
    
    portal._updateProperty('left_slots', lportlets)
    
def addCacheManager(self, out):
    """ adds and associates RAM Cache Manager to portlet """
    cache_manager_id = config.CACHE_MANAGER
    if cache_manager_id not in self.objectIds():
        self.manage_addProduct['StandardCacheManagers'].manage_addRAMCacheManager(cache_manager_id)

    settings={'request_vars': ('AUTHENTICATED_USER',),
              'threshold': 100,
              'max_age': 60,
              'cleanup_interval': 300}
    self[cache_manager_id].manage_editProps('TagCloudCache', settings)

    print >> out, 'Cache Manager created.\n'

def install(self) :
    outStream = StringIO()
    skinsTool = getToolByName(self, 'portal_skins')
    addDirectoryViews(skinsTool, 'skins', config.GLOBALS)
    installSubSkin(self, config.SKIN_NAME, outStream)
    #installSubSkin(self, config.SKIN_OVERLOAD_NAME, outStream)
    addTool(self, outStream)
    addPortlet(self, outStream)
    #addActionProvider(self)
    setupCSS(self, outStream)
    installConfiglet(self, outStream)
    addCacheManager(self, outStream)
    return outStream.getvalue()

def uninstall(self) :
    outStream = StringIO()
    uninstallConfiglet(self, outStream)
    removePortlet(self, outStream)
    return outStream.getvalue()
